package java.awt;

import sun.awt.AWTAccessor;
import sun.security.util.SecurityConstants;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import sun.awt.AppContext;
import java.util.Vector;
import sun.awt.HeadlessToolkit;
import sun.awt.SunToolkit;
import java.awt.peer.SystemTrayPeer;

public class SystemTray
{
    private static SystemTray systemTray;
    private int currentIconID;
    private transient SystemTrayPeer peer;
    private static final TrayIcon[] EMPTY_TRAY_ARRAY;
    
    private SystemTray() {
        this.currentIconID = 0;
        this.addNotify();
    }
    
    public static SystemTray getSystemTray() {
        checkSystemTrayAllowed();
        if (GraphicsEnvironment.isHeadless()) {
            throw new HeadlessException();
        }
        initializeSystemTrayIfNeeded();
        if (!isSupported()) {
            throw new UnsupportedOperationException("The system tray is not supported on the current platform.");
        }
        return SystemTray.systemTray;
    }
    
    public static boolean isSupported() {
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        if ((defaultToolkit instanceof SunToolkit)) {
            initializeSystemTrayIfNeeded();
            return ((SunToolkit)defaultToolkit).isTraySupported();
        }
        return defaultToolkit instanceof HeadlessToolkit && ((HeadlessToolkit)defaultToolkit).isTraySupported();
    }
    
    public void add(TrayIcon o) throws AWTException {
        if (o == null) {
            throw new NullPointerException("adding null TrayIcon");
        }
        TrayIcon[] trayIcons = null;
        TrayIcon[] trayIcons2 = null;
        Vector value = null;
        synchronized (this) {
            trayIcons = SystemTray.systemTray.getTrayIcons();
            value = (Vector)AppContext.getAppContext().get(TrayIcon.class);
            if (value == null) {
                value = new Vector(3);
                AppContext.getAppContext().put(TrayIcon.class, value);
            }
            else if (value.contains(o)) {
                throw new IllegalArgumentException("adding TrayIcon that is already added");
            }
            value.add(o);
            trayIcons2 = SystemTray.systemTray.getTrayIcons();
            o.setID(++this.currentIconID);
        }
        try {
            o.addNotify();
        }
        catch (AWTException ex) {
            value.remove(o);
            throw ex;
        }
        this.firePropertyChange("trayIcons", trayIcons, trayIcons2);
        return;
    }
    
    public void remove(TrayIcon o) {
        if (o == null) {
            return;
        }
        TrayIcon[] trayIcons = null;
        TrayIcon[] trayIcons2 = null;
        synchronized (this) {
            trayIcons = SystemTray.systemTray.getTrayIcons();
            Vector<Object> vector = (Vector)AppContext.getAppContext().get(TrayIcon.class);
            if (vector == null || !vector.remove(o)) {
                return;
            }
            o.removeNotify();
            trayIcons2 = SystemTray.systemTray.getTrayIcons();
        }
        this.firePropertyChange("trayIcons", trayIcons, trayIcons2);
        return;
    }
    
    public TrayIcon[] getTrayIcons() {
        Vector<Object> vector = (Vector)AppContext.getAppContext().get(TrayIcon.class);
        if (vector != null) {
            return (TrayIcon[])(TrayIcon[])vector.toArray(new TrayIcon[vector.size()]);
        }
        return SystemTray.EMPTY_TRAY_ARRAY;
    }
    
    public Dimension getTrayIconSize() {
        return this.peer.getTrayIconSize();
    }
    
    public synchronized void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        if (listener == null) {
            return;
        }
        this.getCurrentChangeSupport().addPropertyChangeListener(propertyName, listener);
        return;
    }
    
    public synchronized void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        if (listener == null) {
            return;
        }
        this.getCurrentChangeSupport().removePropertyChangeListener(propertyName, listener);
        return;
    }
    
    public synchronized PropertyChangeListener[] getPropertyChangeListeners(String propertyName) {
        return this.getCurrentChangeSupport().getPropertyChangeListeners(propertyName);
    }
    
    private void firePropertyChange(String propertyName, Object oldValue, Object o) {
        if (oldValue != null && o != null && oldValue.equals(o)) {
            return;
        }
        this.getCurrentChangeSupport().firePropertyChange(propertyName, oldValue, o);
        return;
    }
    
    private synchronized PropertyChangeSupport getCurrentChangeSupport() {
        PropertyChangeSupport value = (PropertyChangeSupport)AppContext.getAppContext().get(SystemTray.class);
        if (value == null) {
            value = new PropertyChangeSupport(this);
            AppContext.getAppContext().put(SystemTray.class, value);
            return value;
        }
        return value;
    }
    
    synchronized void addNotify() {
        if (this.peer == null) {
            Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
            if ((defaultToolkit instanceof SunToolkit)) {
                this.peer = ((SunToolkit)Toolkit.getDefaultToolkit()).createSystemTray(this);
            }
            else if ((defaultToolkit instanceof HeadlessToolkit)) {
                this.peer = ((HeadlessToolkit)Toolkit.getDefaultToolkit()).createSystemTray(this);
            }
        }
    }
    
    static void checkSystemTrayAllowed() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(SecurityConstants.AWT.ACCESS_SYSTEM_TRAY_PERMISSION);
        }
    }
    
    private static void initializeSystemTrayIfNeeded() {
        synchronized (SystemTray.class) {
            if (SystemTray.systemTray == null) {
                SystemTray.systemTray = new SystemTray();
            }
        }
    }
    
    static void access$000(SystemTray systemTray, String s, Object o, Object o2) {
        systemTray.firePropertyChange(s, o, o2);
    }
    
    static {
        EMPTY_TRAY_ARRAY = new TrayIcon[0];
        AWTAccessor.setSystemTrayAccessor(new AWTAccessor.SystemTrayAccessor() {
            @Override
            public void firePropertyChange(SystemTray systemTray, String s, Object o, Object o2) {
                SystemTray.access$000(systemTray, s, o, o2);
            }
        });
    }
}
